; example2.nsi
;
; This script is based on example1.nsi, but it remember the directory, 
; has uninstall support and (optionally) installs start menu shortcuts.
;
; It will install makensisw.exe into a directory that the user selects,

;--------------------------------

; The name of the installer
Name "DiceMan"

; The file to write
OutFile "DiceManInstall.exe"

; The default installation directory
InstallDir $PROGRAMFILES\DiceMan

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKLM "Software\DiceMan" "Install_Dir"

;--------------------------------

; Pages

Page components
Page directory
Page instfiles

UninstPage uninstConfirm
UninstPage instfiles

;--------------------------------

; The stuff to install
Section "DiceMan Game Files (required)"

  SectionIn RO
  
  CreateDirectory "$INSTDIR\gfx"
  CreateDirectory "$INSTDIR\levels"
  CreateDirectory "$INSTDIR\scrn"
  CreateDirectory "$INSTDIR\sfx"

  ; Set output path to the installation directory.
  SetOutPath $INSTDIR
  
  ; Put file there
  File "DiceMan.exe"
  File "*.dll"
  File "readme.html"

  SetOutPath $INSTDIR\gfx
  File "gfx\*.bmp"
  File "gfx\*.jpg"
  SetOutPath $INSTDIR\levels
  File "levels\*.map"
  SetOutPath $INSTDIR\sfx
  File "sfx\*.wav"

  SetOutPath $INSTDIR
  
  ; Write the installation path into the registry
  WriteRegStr HKLM SOFTWARE\DiceMan "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DiceMan" "DisplayName" "DiceMan"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DiceMan" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DiceMan" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DiceMan" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  
SectionEnd

Section "Source Code"

  SetOutPath "$INSTDIR"
  File "*.cpp"
  File "*.h"
  File "DiceMan.ds*"
  File "DiceMan.opt"
  File "DiceMan.plg"
  File "DiceMan.nsi"
  File "Script1.*"
  File "icon.ico"

SectionEnd

; Optional section (can be disabled by the user)
Section "Start Menu Shortcuts"

  CreateDirectory "$SMPROGRAMS\DiceMan"
  CreateShortCut "$SMPROGRAMS\DiceMan\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\DiceMan\DiceMan Original.lnk" "$INSTDIR\DiceMan.exe" "" "$INSTDIR\DiceMan.exe" 0
  CreateShortCut "$SMPROGRAMS\DiceMan\View Readme.lnk" "$INSTDIR\readme.html" "" "$INSTDIR\readme.html" 0
  
SectionEnd

;--------------------------------

; Uninstaller

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\DiceMan"
  DeleteRegKey HKLM SOFTWARE\DudleySoft\DiceMan

  ; Remove files and uninstaller
  Delete $INSTDIR\*.*

  Delete $INSTDIR\gfx\*.*
  Delete $INSTDIR\levels\*.*
  Delete $INSTDIR\scrn\*.*
  Delete $INSTDIR\sfx\*.*

  RMDir $INSTDIR\music
  RMDir $INSTDIR\gfx
  RMDir $INSTDIR\levels
  RMDir $INSTDIR\scrn
  RMDir $INSTDIR\sfx
  

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\DiceMan\*.*"

  ; Remove directories used
  RMDir "$SMPROGRAMS\DiceMan"
  RMDir "$INSTDIR"

SectionEnd