// File:	DiceMan\\Game.cpp
// Purpose:	Lumdum Dare 48 Hour Contest Game
// Theme:	Random
// Author:	James Watson

#include "diceman.h"

GameMode GAME_Info={GAME_Init,GAME_Update,GAME_Render,GAME_Close};
GameMode GAME2_Info={GAME_ReInit,GAME_Update,GAME_Render,GAME_Close};

Bmp *panel;
Bmp *platforms;
Bmp *borders;
Bmp *player;
Bmp *dice;
Bmp *paneldice;
Bmp *landscape;

int g_entry,g_exit;

int g_playerX,g_playerY;
int g_playerHead,g_playerFrame;
int g_playerDir;
int g_score;
int g_rerolls[4];
int g_lives;
int g_fallRate;
int g_headSpin;
bool g_Fall;
int g_level;
int g_msgTimer;
char g_msg[32];
int g_deadTimer;
bool g_levelDone;


int keyMap;

char *map;
char *contents;

int g_mapWidth;
int g_scroll;
int g_Dead;

int GAME_dice()
{
	return rand()%6;
}

void GAME_LoadMap(int level)
{
	char str[32];
	FILE *fhand;
	short w,h;
	int i,y;

	sprintf(str,"levels/level%d.map",level);

	fhand=fopen(str,"rb");
	fread(&w,1,2,fhand);
	fread(&h,1,2,fhand);

	g_mapWidth=w;

	if (map)
	{
		delete map;
		map=NULL;
		delete contents;
		contents=NULL;
	}

	map=new char[g_mapWidth*h];

	fread(map,g_mapWidth,h,fhand);

	contents=new char[g_mapWidth*h];

	fread(contents,g_mapWidth,h,fhand);

	ENTITY_CleanUp();

	for(i=0;i<g_mapWidth;i++)
	{
		for(y=0;y<6;y++)
		{
			switch(contents[(y*g_mapWidth)+i])
			{
			case 10:	// Player

				g_playerX=i*80+40;
				g_playerY=(y*80)+72;

				if (map[(y*g_mapWidth)+i])
					g_Fall=false,g_fallRate=0;
				else
					g_Fall=true,g_fallRate=0;

				// Clear this so it doesn't cause a problem later
				contents[(y*g_mapWidth)+i]=0;
				break;
			case 11:	// Enemy 1
				ENTITY_Create((i*80)+40,(y*80)+72,0);
				break;
			case 12:
				ENTITY_Create((i*80)+40,(y*80)+72,1);
				break;
			case 13:
				ENTITY_Create((i*80)+40,(y*80)+72,2);
				break;
			}
		}
	}

	for(i=0;i<4;i++)
		g_rerolls[i]=i==3?3:0;

	sprintf(g_msg,"Level %d",g_level);
	g_msgTimer=30;

	g_Dead=false;
	g_levelDone=false;
}

void GAME_UnloadMap()
{
	if (map)
	{
		delete map;
		map=NULL;
		delete contents;
		contents=NULL;
	}
}

void GAME_Init(void)
{

	panel=BMP_Load("gfx/panel.bmp",BMP_NONE);
	platforms=BMP_LoadSprite("gfx/platforms.bmp",80,80);
	borders=BMP_LoadSprite("gfx/borders.bmp",16,80);
	player=BMP_LoadSprite("gfx/player.bmp",32,48);
	dice=BMP_LoadSprite("gfx/dice.bmp",16,16);
	paneldice=BMP_LoadSprite("gfx/paneldice.bmp",32,32);
	landscape=BMP_Load("gfx/landscape.bmp",BMP_NONE);

	g_playerX=0;
	g_playerY=g_entry*80+8;

	g_playerHead=GAME_dice();
	g_playerFrame=0;
	g_playerDir=0;

	keyMap=0;

	g_score=0;

	
	g_lives=3;
	g_headSpin=20;

	g_scroll=0;

	map=NULL;
	contents=NULL;
	g_level=1;

	ENTITY_Init();

	PARTICLE_Init();

	GAME_LoadMap(g_level);
}

void GAME_ReInit(void)
{
	int i;

	panel=BMP_Load("gfx/panel.bmp",BMP_NONE);
	platforms=BMP_LoadSprite("gfx/platforms.bmp",80,80);
	borders=BMP_LoadSprite("gfx/borders.bmp",16,80);
	player=BMP_LoadSprite("gfx/player.bmp",32,48);
	dice=BMP_LoadSprite("gfx/dice.bmp",16,16);
	paneldice=BMP_LoadSprite("gfx/paneldice.bmp",32,32);
	landscape=BMP_Load("gfx/landscape.bmp",BMP_NONE);

	g_playerHead=GAME_dice();
	g_playerFrame=0;
	g_playerDir=0;

	keyMap=0;

	for(i=0;i<4;i++)
		g_rerolls[i]=0;
	
	g_headSpin=20;

	g_scroll=0;

	map=NULL;
	contents=NULL;
	g_level++;

	ENTITY_Init();

	PARTICLE_Init();

	GAME_LoadMap(g_level);
}


void GAME_SkyRender()
{
	SDL_Rect rect;

	rect.x=g_scroll*128/((g_mapWidth*80)-512);
	rect.y=0;
	rect.w=512;
	rect.h=480;

	SDL_BlitSurface(landscape->bmp,&rect,g_MainWindow,NULL);
}

void GAME_RenderPlayer()
{
	int frm[]={0,1,2,3,2,1};

	BMP_RenderSprite(player,g_playerX-16-g_scroll,g_playerY-48,g_playerDir,frm[g_playerFrame]);
	BMP_RenderSprite(dice,g_playerX-8-g_scroll,g_playerY-66,3,g_playerHead);

}

void GAME_RenderPanel()
{
	char str[8];
	int i,x;

	BMP_Render(panel,512,0,BMP_NONE);
	BMP_Render(g_SmallLogo,576,8,BMP_HCENTER);

	FONT_Render(g_Font,576,100,"Score",FONT_HCENTER);
	sprintf(str,"%06d",g_score);
	FONT_Render(g_DarkFont,576,124,str,FONT_HCENTER);

	FONT_Render(g_Font,576,164,"Lives",FONT_HCENTER);

	x=578-10*g_lives;
	for(i=0;i<g_lives;i++)
		BMP_RenderSprite(dice,x+20*i,184,2,i);

	if (g_rerolls[0])
	{
		BMP_RenderSprite(paneldice,540,232,0,g_rerolls[0]-1);
		FONT_Render(g_Font,580,240,"F1",FONT_NORMAL);
	}
	FONT_Render(g_Font,576,266,"- Platform",FONT_HCENTER);

	if (g_rerolls[1])
	{
		BMP_RenderSprite(paneldice,540,286,1,g_rerolls[1]-1);
		FONT_Render(g_Font,580,294,"F2",FONT_NORMAL);
	}
	FONT_Render(g_Font,576,320,"+ Platform",FONT_HCENTER);
 
	if (g_rerolls[2])
	{
		BMP_RenderSprite(paneldice,540,340,2,g_rerolls[2]-1);
		FONT_Render(g_Font,580,348,"F3",FONT_NORMAL);
	}
	FONT_Render(g_Font,576,374,"- Enemy",FONT_HCENTER);

	BMP_RenderSprite(paneldice,528,420,3,g_playerHead);
	FONT_Render(g_Font,592,418,"Rolls",FONT_HCENTER);

	sprintf(str,"%d",g_rerolls[3]);
	FONT_Render(g_DarkFont,592,438,str,FONT_HCENTER);
}

void GAME_RenderMap(int scroll)
{
	int xoff=scroll%80;
	int xpos=scroll/80;

	int x,y;

	x=-xoff;

	while(x<512)
	{
		for(y=0;y<6;y++)
		{
			if (map[(y*g_mapWidth)+xpos]>0)
				BMP_RenderSprite(platforms,x,y*80,0,map[(y*g_mapWidth)+xpos]-1);
			if (contents[(y*g_mapWidth)+xpos]>0)
				BMP_RenderSprite(dice,x+32,y*80+56,contents[(y*g_mapWidth)+xpos]-7,y);
		}

		xpos++;
		x+=80;
	}
}

void GAME_Render(void)
{
	GAME_SkyRender();

	GAME_RenderPlayer();

	ENTITY_Render();

	GAME_RenderMap(g_scroll);

	PARTICLE_Render();

	GAME_RenderPanel();


	if (g_msgTimer)
	{
		FONT_Render(g_BigFont,256,240,g_msg,FONT_HCENTER|FONT_VCENTER);
		g_msgTimer--;
	}
	
}

void GAME_RollDice(int colour)
{
	int x=g_playerX/80;

	// Stop you rolling a dice you dont have.
	if (g_rerolls[colour]==0)
		return;

	if (g_playerDir==0)
		x++;
	else
		x--;

	if (x<0) 
		x=0;
	if (x>=g_mapWidth)
		x=g_mapWidth-1;

	switch(colour)
	{
	case	DICE_RED:
		// Clear the map
		switch(map[(g_playerHead*g_mapWidth)+x])
		{
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			PARTICLE_Create((x*80+20),(g_playerHead*80)+20,3);
			PARTICLE_Create((x*80+60),(g_playerHead*80)+20,3);
			PARTICLE_Create((x*80+40),(g_playerHead*80)+40,3);
			PARTICLE_Create((x*80+20),(g_playerHead*80)+60,3);
			PARTICLE_Create((x*80+60),(g_playerHead*80)+60,3);
			MAIN_PlaySfx(dicethrow);
			MAIN_PlaySfx(explode);
			map[(g_playerHead*g_mapWidth)+x]=0;
			g_rerolls[0]--;
			g_headSpin=20;
			g_score+=100;
			break;
		}
		break;

	case	DICE_GREEN:
		if (map[(g_playerHead*g_mapWidth)+x]==0)
		{
			PARTICLE_Create((x*80+20),(g_playerHead*80)+76,0);
			PARTICLE_Create((x*80+40),(g_playerHead*80)+76,0);
			PARTICLE_Create((x*80+60),(g_playerHead*80)+76,0);
			MAIN_PlaySfx(dicethrow);
			MAIN_PlaySfx(footfall);
			map[(g_playerHead*g_mapWidth)+x]=1;
			g_headSpin=20;
			g_rerolls[1]--;
			g_score+=150;
		}
		break;

	case DICE_BLUE:
		// Remove enemy when enemies are in
		{
			Entity *p;
			int x=0;
			while(x<360)
			{
				// Check collide ahead of the line
				p=ENTITY_Collide((g_playerDir==1)?(g_playerX-x):(g_playerX+x),(g_playerHead*80)+72);
				if (p)
				{
					g_score+=(p->type+1)*100;
					PARTICLE_Create(p->x,p->y-24,1);
					ENTITY_Destroy(p);
					MAIN_PlaySfx(explode);
					g_headSpin=20;
					g_rerolls[2]--;
					return;
				}
				x+=40;
			}
		}
		break;

	case DICE_WHITE:
		g_headSpin=20;
		g_rerolls[3]--;
		break;
	}

}



void GAME_PlayerRise(int spd)
{
	int x,y;

	x=g_playerX/80;

	g_playerY+=spd;
	
	y=(g_playerY-64)/80;

	if (y<0)
		return;

	if (map[(y*g_mapWidth)+x]==5)
		g_playerY=(y*80)+64+80;
}

void GAME_PlayerFall(int spd)
{
	int x,y,y0;

	x=g_playerX/80;

	y0=g_playerY/80;

	g_playerY+=spd;

	y=g_playerY/80;

	if (g_Dead)
	{
		return;
	}

	if (y>6)
	{
		g_Dead=true;
		g_deadTimer=120;
		MAIN_PlaySfx(scream);
		return;
	}

	if (map[(y0*g_mapWidth)+x]!=0)
	{
		if (g_playerY>((y0*80)+72))
		{
			g_playerY=((y0*80)+72),g_Fall=false;
			MAIN_PlaySfx(footfall);
			return;
		}
	}
	if (map[(y*g_mapWidth)+x]==5)
	{
		g_playerY=(y*80)-2,g_Fall=false;
		MAIN_PlaySfx(footfall);
	}
}

void GAME_PlayerGoLeft(int spd)
{
	int x,y,x0;

	x=g_playerX/80;

	// Block under left edge plus update position
	x0=(g_playerX-15+spd)/80;
	// Quick stop for going off the left of the map
	if (x0<0) return;

	g_playerDir=1;

	y=g_playerY/80;

	if ((y<0)||(y>5))
		return;

	switch(map[(y*g_mapWidth)+x])
	{
	case 0:
	case 1:
	case 3:
	case 5:
		if (x0==x)
		{
			g_playerX+=spd;
			g_playerFrame=(g_playerFrame+1)%6;
		}
		else
		{
			switch(map[(y*g_mapWidth)+x0])
			{
			case 0:
			case 1:
			case 2:
				g_playerX+=spd;
				g_playerFrame=(g_playerFrame+1)%6;
				// Move us back up if we came off block 5
				if ((g_playerY-(80*y))>72)
					g_playerY=(80*y)+72;
				break;
			}
		}
		break;
	case 2:
	case 4:
		// Check if we're within 8 pixels of the left of these blocks
		// if not then we can move left.
		if (g_playerX>(x*80+20))
		{
			g_playerFrame=(g_playerFrame+1)%6;
			g_playerX+=spd;
		}
		break;
	}

	// Calculate new position
	x=g_playerX/80;
	y=(g_playerY-8)/80;	// Bodge to make it fall when standing on solid block
	// Check to see if we're supposed to fall
	if (map[(y*g_mapWidth)+x]==0)
	{
		if (!g_Fall)
			g_Fall=true,g_fallRate=0;
	}
}

void GAME_PlayerGoRight(int spd)
{
	int x,y,x0;

	x=g_playerX/80;

	// Block under then right edge plus update position
	x0=(g_playerX+15+spd)/80;
	// Quick stop for going off the right of the map
	if (x0>(80*g_mapWidth))
		return;

	g_playerDir=0;
   
	y=g_playerY/80;

	if ((y<0)||(y>5))
		return;

	switch(map[(y*g_mapWidth)+x])
	{
	case 0:
	case 1:
	case 2:
	case 5:	// Move out of the impassible block
		if (x0==x)
		{
			g_playerX+=spd;
			g_playerFrame=(g_playerFrame+1)%6;
		}
		else
		{
			switch(map[(y*g_mapWidth)+x0])
			{
			case 0:
			case 1:
			case 3:
			case 6:	// Exit
				g_playerX+=spd;
				g_playerFrame=(g_playerFrame+1)%6;

				// Move us back up if we came off block 5
				if ((g_playerY-(80*y))>72)
					g_playerY=(80*y)+72;
				break;
			}
		}
		break;

	case 3:
	case 4:
	case 6:
		// Check if we're within 8 pixels of the left of these blocks
		// if not then we can move left.
		if (g_playerX<(x*80+60))
		{
			g_playerFrame=(g_playerFrame+1)%6;
			g_playerX+=spd;
		}
		else if (map[(y*g_mapWidth)+x]==6)
		{
			g_levelDone=true;
		}
		break;
	}
   
	// Calculate new position
	x=g_playerX/80;
	y=(g_playerY-8)/80;
	// Check to see if we're supposed to fall
	if (map[(y*g_mapWidth)+x]==0)
	{
		if (!g_Fall)
			g_Fall=true,g_fallRate=0;
	}
}

void GAME_Pickup()
{
	int x,y,cx,cy;

	x=g_playerX/80;
	y=g_playerY/80;

	if (contents[(y*g_mapWidth)+x])
	{
		cx=-(x*80+40)+(g_playerX-16);
		cy=-(y*80+56)+(g_playerY-64);

		if ( (cx>-32) && (cx<16) &&
			(cy>-64) && (cy<16) )
		{
			switch(contents[(y*g_mapWidth)+x])
			{
			case 7:
				if (g_rerolls[0]<6)
				{
					PARTICLE_Create(g_playerX,g_playerY-16,2);
					MAIN_PlaySfx(knock);
					g_rerolls[0]++;
					g_rerolls[3]++;
					g_score+=50;
					contents[(y*g_mapWidth)+x]=0;
				}
				break;
			case 8:
				if (g_rerolls[1]<6)
				{
					PARTICLE_Create(g_playerX,g_playerY-16,2);
					MAIN_PlaySfx(knock);
					g_rerolls[1]++;
					g_rerolls[3]++;
					g_score+=50;
					contents[(y*g_mapWidth)+x]=0;
				}
				break;
			case 9:
				if (g_rerolls[2]<6)
				{
					PARTICLE_Create(g_playerX,g_playerY-16,2);
					MAIN_PlaySfx(knock);
					g_rerolls[2]++;
					g_rerolls[3]++;
					g_score+=50;
					contents[(y*g_mapWidth)+x]=0;
				}
				break;
			}
		}
	}
}

void GAME_PlayerUpdate()
{
	if (g_Fall)
	{
		if (g_fallRate<0)
		{
			GAME_PlayerRise(g_fallRate);
		}
		else
		{
			GAME_PlayerFall(g_fallRate);
		}

		if (g_Fall)
		{
			if (g_fallRate<16)
				g_fallRate+=2;
		}
	}

	// Dont do the rest of this if the player is dead
	if (g_Dead)
		return;

	if (g_headSpin)
	{
		g_headSpin--;
		g_playerHead=GAME_dice();
	}

	GAME_Pickup();

	if (ENTITY_Collide(g_playerX,g_playerY))
	{
		if (!g_Fall)
			g_Fall=true,g_fallRate=0;
		g_Dead=true;
		PARTICLE_Create(g_playerX,g_playerY-32,1);
		MAIN_PlaySfx(scream);
	}
}

bool GAME_PlayerCanDrop()
{
	int x,y;

	x=g_playerX/80;
	y=(g_playerY+8)/80;

	if (y>5) return false;

	if (map[(y*g_mapWidth)+x]==5)
		return false;
	return true;
}

bool GAME_Update(void)
{
	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				if (!g_Fall)
				{
					g_Fall=true;
					g_fallRate=0;
				}
				g_Dead=true;
				g_deadTimer=60;
				break;
			case SDLK_LEFT:
				keyMap|=KEY_LEFT;
				break;
			case SDLK_RIGHT:
				keyMap|=KEY_RIGHT;
				break;
			case SDLK_UP:
				keyMap|=KEY_UP;
				break;
			case SDLK_DOWN:
				keyMap|=KEY_DOWN;
				break;
			case SDLK_SPACE:
				if (g_Dead)
				{
					g_deadTimer=1;
				}
				else
					GAME_RollDice(DICE_WHITE);
				break;
			case SDLK_F1:
				GAME_RollDice(DICE_RED);
				break;
			case SDLK_F2:
				GAME_RollDice(DICE_GREEN);
				break;
			case SDLK_F3:
				GAME_RollDice(DICE_BLUE);
				break;
			}
			break;

		case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				break;
			case SDLK_LEFT:
				keyMap&=~KEY_LEFT;
				break;
			case SDLK_RIGHT:
				keyMap&=~KEY_RIGHT;
				break;
			case SDLK_UP:
				keyMap&=~KEY_UP;
				break;
			case SDLK_DOWN:
				keyMap&=~KEY_DOWN;
				break;
			case SDLK_SPACE:
				keyMap&=~KEY_FIRE;
				break;
			}
			break;
		}
	}

	ENTITY_Update();

	if (!g_Dead)
	{

		if (keyMap & KEY_LEFT)
		{
			GAME_PlayerGoLeft(-4);
		}
		if (keyMap & KEY_RIGHT)
		{
			GAME_PlayerGoRight(4);
		}

		if ((keyMap & KEY_UP) && (g_Fall==false))
		{
			g_Fall=true;
			g_fallRate=-20;
		}

		if ((keyMap & KEY_DOWN) && (g_Fall==false))
		{
			g_Fall=true;
			g_fallRate=0;
			if (GAME_PlayerCanDrop())
			// Drop us down through the platform
			g_playerY+=8;

			// Stop this key repeating
			keyMap &=~KEY_DOWN;
		}

		GAME_PlayerUpdate();

		g_scroll=g_playerX-240;

		if (g_scroll<0)
			g_scroll=0;

		if (g_scroll>((80*g_mapWidth)-512))
			g_scroll=((80*g_mapWidth)-512);
	}
	else
	{
		GAME_PlayerUpdate();

		g_msgTimer=30;
		sprintf(g_msg,"You Died!");
		g_deadTimer--;

		if (g_deadTimer==0)
		{
			g_lives--;
			if (g_lives==0)
			{
				MODE_Set(MODE_GAMEOVER);
			}
			else
				GAME_LoadMap(g_level);
		}
	}

	if (g_levelDone)
	{
		MODE_Set(MODE_FINISH);
	}
	return false;
}

void GAME_Close(void)
{
	BMP_Free(panel);
	BMP_Free(platforms);
	BMP_Free(borders);
	BMP_Free(player);
	BMP_Free(dice);
	BMP_Free(paneldice);
	BMP_Free(landscape);

	ENTITY_Close();
	PARTICLE_Close();
	GAME_UnloadMap();
}