// File:	DiceMan\\Main.cpp
// Purpose:	Lumdum Dare 48 Hour Contest Game
// Theme:	Random
// Author:	James Watson

#include "diceman.h"

SDL_Surface *g_MainWindow;

int g_sfxVol;

Mix_Music *music;
Mix_Chunk *knock;
Mix_Chunk *scream;
Mix_Chunk *footfall;
Mix_Chunk *explode;
Mix_Chunk *dicethrow;


GameMode *g_Modes[]={
	&TITLE_Info,
	&GAME_Info,
	&OPTIONS_Info,
	&HELP_Info,
	&GAMEOVER_Info,
	&FINISH_Info,
	&GAME2_Info,
	&HIGHSCORE_Info,
	&GAMEEND_Info
};

HighScore g_high[5];

GameMode *g_Mode;

void MAIN_init()
{
	int i;
	FILE *fhand;

	MODE_Set(MODE_TITLE);

	fhand=fopen("diceman.ini","rb");
	if (fhand)
	{
		fread(&g_sfxVol,1,4,fhand);
		fread(g_high,5,sizeof(HighScore),fhand);
		fclose(fhand);
	}
	else
	{
		g_sfxVol=9;

		for(i=0;i<5;i++)
		{
			g_high[i].score=25000-(i*5000);
			strcpy(g_high[i].name,(i&1)?"Dudley":"DiceMan");
		}
	}
	knock=Mix_LoadWAV("sfx/knock.wav");    
	scream=Mix_LoadWAV("sfx/scream.wav");   
	footfall=Mix_LoadWAV("sfx/footfall.wav");
	explode=Mix_LoadWAV("sfx/explode.wav");
	dicethrow=Mix_LoadWAV("sfx/dice.wav");

}

void MAIN_PlaySfx(Mix_Chunk *sfx)
{
	int ch=Mix_PlayChannel(-1,sfx,0);

	Mix_Volume(ch,14*g_sfxVol);
}

void MODE_Set(int mode)
{
	if (g_Mode)
		g_Mode->Close();

	g_Mode=g_Modes[mode];

	g_Mode->Init();
}

void MAIN_Close()
{
	FILE *fhand;

	fhand=fopen("diceman.ini","wb");
	fwrite(&g_sfxVol,1,4,fhand);

	fwrite(g_high,5,sizeof(HighScore),fhand);

	fclose(fhand);
}

int main(int argc,char *argv[])
{
	bool quit=false;

	g_Mode=NULL;


	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO);

	SDL_ShowCursor(SDL_DISABLE);

	//g_MainWindow=SDL_SetVideoMode(640,480,24,/*SDL_FULLSCREEN|*/SDL_DOUBLEBUF|SDL_HWPALETTE);
	g_MainWindow=SDL_SetVideoMode(640,480,24,SDL_FULLSCREEN|SDL_DOUBLEBUF|SDL_HWPALETTE);
	SDL_WM_SetCaption("DiceMan in The DiceMan Cometh",NULL);

	if (g_MainWindow==NULL)
	{
		return 1;
	}

	Mix_OpenAudio(44100,AUDIO_S16SYS,2,4096);

	MAIN_init();

	while(!quit)
	{
		g_Mode->Render();
		quit=g_Mode->Update();
		
		// Throw away a random number, this will make our randomness
		// even more so cause we never know what frame we're getting
		// our random number.
		rand();

		SDL_Flip(g_MainWindow);
	}
	g_Mode->Close();

	Mix_CloseAudio();

	MAIN_Close();

	SDL_Quit();
	return 0;
}
