// File:	DiceMan\\Particle.cpp
// Purpose:	Lumdum Dare 48 Hour Contest Game
// Theme:	Random
// Author:	James Watson

#include "diceman.h"

ParticleSystem psystems[32];
Bmp *particles;

void PARTICLE_Init()
{
	int i;

	for(i=0;i<32;i++)
		psystems[i].used=false;

	particles=BMP_LoadSprite("gfx/particles.bmp",8,8);
}

void PARTICLE_CleanUp()
{
	int i;

	for(i=0;i<32;i++)
		if (psystems[i].used)
			delete psystems[i].pParticles;
}

void PARTICLE_Close()
{
	PARTICLE_CleanUp();
	BMP_Free(particles);
}

void PARTICLE_Create(int x,int y,int type)
{
	int i;
	ParticleSystem *p=NULL;

	for(i=0;i<32;i++)
		if (psystems[i].used==false)
			p=&psystems[i],i=32;

	if (p==NULL)
		return;

	p->used=true;

	if (type & PARTICLE_FIREWORK)
		p->life=48+GAME_dice();
	else
		p->life=24+GAME_dice();

	p->particles=6+GAME_dice();

	p->pParticles=new Particle[p->particles];

	for(i=0;i<p->particles;i++)
	{
		p->pParticles[i].type=type&3;
		p->pParticles[i].frame=0;
		p->pParticles[i].x=(x+(4*GAME_dice())-12)<<4;
		p->pParticles[i].y=(y+(4*GAME_dice())-12)<<4;
		
		if (type & PARTICLE_FIREWORK)
		{
			p->pParticles[i].xv=(rand()%31)-16;
			p->pParticles[i].yv=-rand()%16;
		}
		else
		{
			p->pParticles[i].xv=(GAME_dice()+GAME_dice())-7;
			p->pParticles[i].yv=-GAME_dice();
		}

		p->pParticles[i].xa=0;
		p->pParticles[i].ya=2;
	}
}

void PARTICLE_delete(ParticleSystem *ps)
{
	delete ps->pParticles;
	ps->used=false;
}

void PARTICLE_RenderSystem(ParticleSystem *ps)
{
	int i;
	Particle *p=ps->pParticles;

	for(i=0;i<ps->particles;i++)
	{
		p->frame=(p->frame+1)%16;
		BMP_RenderSprite(particles,(p->x>>4)-4-g_scroll,(p->y>>4)-4,p->type,p->frame>>2);
		p->x+=p->xv;
		if (p->xv<64)
			p->xv+=p->xa;
		p->y+=p->yv;
		if (p->yv<64)
			p->yv+=p->ya;
		p++;
	}

	ps->life--;

	if (ps->life==0)
		PARTICLE_delete(ps);
}

void PARTICLE_Render()
{
	int i;

	for(i=0;i<32;i++)
		if (psystems[i].used)
			PARTICLE_RenderSystem(&psystems[i]);
}