// File:	DiceMan\\Title.cpp
// Purpose:	Lumdum Dare 48 Hour Contest Game
// Theme:	Random
// Author:	James Watson

#include "diceman.h"

GameMode TITLE_Info={TITLE_Init,TITLE_Update,TITLE_Render,TITLE_Close};

Bmp *logo,*g_SmallLogo=NULL;
Font *g_Font=NULL,*g_DarkFont=NULL;
Font *g_BigFont=NULL,*g_BigDarkFont=NULL;
int bmp_y,bmp_spd;
int initial_delay=60;
int bRenderMenu;
int menuOpt;

void TITLE_Init(void)
{
	if (g_SmallLogo==NULL)
	{
		g_SmallLogo=BMP_Load("gfx/logos.bmp",BMP_COLORKEY);
		g_Font=FONT_Load("gfx/font.bmp",0);
		g_DarkFont=FONT_Load("gfx/darkfont.bmp",0);
		g_BigFont=FONT_Load("gfx/bigfont.bmp",2);
		g_BigDarkFont=FONT_Load("gfx/bigdarkfont.bmp",2);
	}

	logo=BMP_Load("gfx/logo.bmp",BMP_COLORKEY);

 	bmp_y=-160;
	bmp_spd=0;
	bRenderMenu=false;
	menuOpt=0;

	PARTICLE_Init();
	g_scroll=0;
}

char *menuStr[]={"Start Game","Options","Help","High Scores","Quit"};

void TITLE_Background(void)
{
	SDL_Rect rect;
	int i;

	rect.x=0;
	rect.w=640;
	rect.y=0;
	rect.h=2;

	for(i=0;i<240;i++)
	{
		SDL_FillRect(g_MainWindow,&rect,SDL_MapRGB(g_MainWindow->format,i+8,0,0));
		rect.y+=2;
	}

	FONT_Render(g_Font,640,480,"Version "VERSION_MAIN"."VERSION_REV,FONT_RIGHT|FONT_BOTTOM);
}

void TITLE_Particle()
{
	int w=FONT_Length(g_BigFont,menuStr[menuOpt])/2;

	PARTICLE_Create(320-w,232+36*menuOpt,2);
	PARTICLE_Create(320+w,232+36*menuOpt,2);
}

void TITLE_Render(void)
{
	int i;
	static int frame=0;

	TITLE_Background();

	BMP_Render(logo,320,bmp_y,BMP_HCENTER);

	if (bRenderMenu)
	{
		for(i=0;i<5;i++)
			FONT_Render(menuOpt==i?g_BigFont:g_BigDarkFont,320,216+36*i,menuStr[i],FONT_HCENTER);
	}

	FONT_RenderSpecial(g_BigFont,g_BigDarkFont,320,400,"Ludum Dare 48 Hour Contest",(frame>>2)%64,FONT_HCENTER);
	FONT_RenderSpecial(g_DarkFont,g_Font,320,440,"by James Watson (DudleySoft)",96-(frame>>2)%96,FONT_HCENTER);

	frame++;

	PARTICLE_Render();
}

bool TITLE_Update(void)
{
	SDL_Event event;

	// This is a hack for fullscreen which seems to run for about a second without displaying anything
	// when you first run the program.
	if (initial_delay)
	{
		initial_delay--;
		return false;
	}

	bmp_y+=bmp_spd;
	
	bmp_spd++;

	if (bmp_y>=32)
	{
		bmp_y=32;

		if (bmp_spd>4)
		{
			PARTICLE_Create(240+(rand()%40),192,3);
			PARTICLE_Create(280+(rand()%80),192,3);
			PARTICLE_Create(360+(rand()%40),192,3);
			MAIN_PlaySfx(footfall);
			bmp_spd=(-bmp_spd)>>1;
		}
		else
		{
			bRenderMenu=true;
			bmp_spd=0;
		}
	}


	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				MAIN_PlaySfx(dicethrow);
				return true;
			case SDLK_UP:
				MAIN_PlaySfx(knock);
				menuOpt=(menuOpt+4)%5;
				TITLE_Particle();
				break;
			case SDLK_DOWN:
				MAIN_PlaySfx(knock);
				menuOpt=(menuOpt+1)%5;
				TITLE_Particle();
				break;
			case SDLK_RETURN:
				MAIN_PlaySfx(dicethrow);
				switch(menuOpt)
				{
				case 0:
					MODE_Set(MODE_GAME);
					break;
				case 1:
					MODE_Set(MODE_OPTIONS);
					break;
				case 2:
					MODE_Set(MODE_HELP);
					break;
				case 3:
					g_scoreEntry=false;
					MODE_Set(MODE_HIGHSCORE);
					break;
				case 4:
					return true;
				}
				break;
			}
		}
	}

	return false;

}

void TITLE_Close(void)
{
	PARTICLE_Close();
	BMP_Free(logo);
}